/*
 * Copyright 2018-Present The CloudEvents Authors
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package io.cloudevents;

/**
 * Interface that defines a wrapper for CloudEvent data.
 * <p>
 * This interface can be overridden to include any type of data inside a {@link CloudEvent}, given it has a method to convert back to bytes.
 */
public interface CloudEventData {

    /**
     * Returns the bytes representation of this data instance.
     * <p>
     * Note: depending on the implementation, this operation may be expensive.
     *
     * @return this data, represented as bytes.
     */
    byte[] toBytes();

}
